#!/usr/bin/perl -w
###############################################################################
#
# Freenet CGI Request Client v0.0.5
#
# Copyright (C) 1999, 2000 Steven Hazel
#
# Edited May 26, 2000 by Lee Benjamin Burhans
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# see http://freenet.sourceforge.net/ for more info on Freenet
#
###############################################################################

use strict;
use IO::Socket;
use CGI qw(:standard);
use Digest::SHA1 qw(sha1_hex);

###############################################################################

my $FREENETPROTO    = "1.1";                        # supported protocol 
my $DEFAULTNODE     = "localhost";                  # def node host 
my $DEFAULTPORT     = 19114;                        # def node port
my $DEFAULTHOPS     = 5;                            # def hops for DataRequest
my $DEFAULTDL       = 5;                            # low value for depth
my $DEFAULTDH       = 30;                           # high value for depth
my $DEFAULTHASH     = "SHA1";                       # hash type

my ($remote, $port, $iaddr, $paddr, $proto, $line, $insert, $request, $myhost,
    $myport, $key, $hop, $message, $id, $pid, $pair, $name, $value, $buffer,
    $messagetype, %fields, $field, $depth, $sendport, $paddr2, $dl, $dh, $uhkey);

###############################################################################

$dh     = $DEFAULTDH;                               # high depth for DataRequest
$dl     = $DEFAULTDL;                               # low depth for DataRequest
$remote = $DEFAULTNODE;                             # node host to contact 
$port   = param('port')     || $DEFAULTPORT;        # node port to contact
$hop    = param('hop')      || $DEFAULTHOPS;        # hops for DataRequest
$key    = $ENV{'PATH_INFO'} || &blurb();            # key to request

$key =~ s/^\///;                                    # kill leading "/"
$key =~ tr/+/ /;                                    # replace "+" with spaces
$key =~ s/%([a-fA-F0-9]{2})/pack("C", hex($1))/eg;  # decode hex

$key || blurb();                                    # display blurb if no key left

#
# store the unhashed key to determine the content type later
#

$uhkey = $key;

#
# convert service name to port, if necessary
#

$port = getservbyname($port, 'tcp') if ($port =~ /\D/);

if (!$port)
    {
    print "Content-type: text/html\n\n";
    print p("No suitable port number was found.");
    exit;
    }

$iaddr = inet_aton($remote);

if (!$iaddr)
    {
    print "Content-type: text/html\n\n";
    print p("No such host: $remote");
    exit;
    }

$paddr = sockaddr_in($port, $iaddr);
$proto = getprotobyname('tcp');

#
# now open the server and send the HandshakeRequest
#

if (!socket(SOCK, PF_INET, SOCK_STREAM, $proto))    
    {
    print "Content-type: text/html\n\n";
    print p("Unable to create socket. ($!)");
    exit;
    }

#
# no buffering on socket
#

my $oldfh = select(SOCK); $| = 1; select($oldfh);

#
# connect to freenet node
#

if (!connect(SOCK, $paddr))
    {
    print "Content-type: text/html\n\n";
    print p("Unable to connect. ($!)");
    exit;
    }

#
# assemble handshake message and send to node
#

$id = mk_random_id();

$message = "HandshakeRequest\n" .
           "UniqueID=$id\n" .
           "Depth=1\n" .
           "HopsToLive=1\n" .
           "KeepAlive=true\n" .
           "EndMessage\n";

print SOCK $message;

#
# get returned message type
#

$messagetype = <SOCK>;

#
# read in message contents
#

while (defined($field = <SOCK>) && ($field =~ /=/)) 
    {
    my($fieldname, $fieldvalue) = split('=', $field);

    chomp $fieldvalue;
    $fields{$fieldname} = $fieldvalue;
    }

#
# returned message should be HandshakeReply
#

if ($messagetype =~ /HandshakeReply/) 
    {
    if ($fields{'Version'} < $FREENETPROTO) 
        {
        print "Content-type: text/html\n\n";
        print p("Server running version $fields{'Version'}, expected $FREENETPROTO or better.");
        exit;
        }
    } 
else 
    {
    print "Content-type: text/html\n\n";
    print p("Received invalid response.");
    exit;
    }

#
# hash key for the request
#

if ($DEFAULTHASH eq 'SHA1') 
    {    
    $key = sha1_hex($key);    
    $key =~ tr/[a-z]/[A-Z]/;
    }

#
# assemble DataRequest message
#

$id = mk_random_id();
$depth = (int(rand($dh - $dl + 1)) + $dl);

$message = "DataRequest\n" .
           "SearchKey=$key\n" . 
           "UniqueID=$id\n" . 
           "Depth=$depth\n" . 
           "HopsToLive=$hop\n" . 
           "KeepAlive=true\n" . 
           "EndMessage\n";

print SOCK $message;

#
# get returned message type
#

$messagetype = <SOCK>;

#
# read message contents
#

%fields = ();

while (defined($field = <SOCK>) && ($field =~ /=/)) 
    {
    my($fieldname, $fieldvalue) = split('=', $field);

    chomp $fieldvalue;
    $fields{$fieldname} = $fieldvalue;
    }

#
# message reply should be DataReply
#

if ($messagetype =~ /DataReply/) 
    {
    if ($fields{'UniqueID'} eq $id) 
        {
        #
        # print contents of found file
        #

        print "Content-type: " . get_content_type($uhkey) . "\n\n";

        while (<SOCK>) 
            {
            print;
            }
        }
    } 
elsif ($messagetype =~ /RequestFailed/) 
    {
    if ($fields{'UniqueID'} eq $id) 
        {
        print "Content-type: text/html\n\n";
        print p("Requested key not found.");
        }
    } 
elsif ($messagetype =~ /TimedOut/) 
    {
    if ($fields{'UniqueID'} eq $id) 
        {
        print "Content-type: text/html\n\n";
        print p("Request timed out.");
        }
    } 
else 
    {
    if ($fields{'UniqueID'} eq $id) 
        {
        print "Content-type: text/html\n\n";
        print p("Received invalid reply.");
        }
    }

#
# terminate connection to node
#

if (!close(SOCK))
    {
    print p("Unable to close socket. ($!)");
    exit;
    }

exit;

###############################################################################

sub blurb()
    {
    print "Content-type: text/html\n\n";
    print p("Freenet CGI Request Client v0.0.5");
    print p("Copyright (C) 1999, 2000 Steven Hazel.");
    print p("Mangled on May 26 2000, by Lee Benjamin Burhans.");
    print p("See <a href=http://freenet.sourceforge.net/>http://freenet.sourceforge.net/</a> for more info on Freenet.");

    exit;
    }

###############################################################################

sub mk_random_id()
    {
    my($id) = ();

    for (1 .. 16) 
        { 
        $id .= sprintf("%x", int(rand(16)));
        }

    return $id;
    }

###############################################################################

sub get_content_type()
    {
    my($key)    = @_;
    my($type)   = ();
    my($def_ct) = "text/plain";

    my(%cont_types) = ("\.html" => "text/html",
                       "\/"     => "text/html",
                       "\.txt"  => "text/plain",
                       "\.gz"   => "application/gzip",
                       "\.tgz"  => "application/gzip",
                       "\.z"    => "application/x-compress",
                       "\.zip"  => "application/x-Zip",
                       "\.jar"  => "application/x-Zip",
                       "\.jpg"  => "image/jpeg",
                       "\.jpeg" => "image/jpeg",
                       "\.gif"  => "image/gif",
                       "\.png"  => "image/png",
                       "\.mp3"  => "audio/x-mpeg3"
                       );

    foreach $type (keys %cont_types)
        {
        if ($key =~ m/$type$/i)
            {
            return $cont_types{$type};
            }
        }    

    return $def_ct;
    }

###############################################################################
